@@ -38,7 +38,10 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <time.h>
+
+#ifndef __MINGW__
 #include <pwd.h>
+#endif
 #include <unistd.h>
 
 /* ---------------------------------------------------------------------- */
@@ -47,34 +50,34 @@ char *latex_print_field(FieldType field, int start);
 
 PrintDescriptionRecord
 LaTeXPrintDescriptionRecord = {
-  "{\\em %d:\\/} ",		/* CPOSPrintFormat */
+  "{\\em %d:\\/} ",                /* CPOSPrintFormat */
 
-  "{\\sf ",			/* BeforePrintStructures */
-  " ",				/* PrintStructureSeparator */
-  "} ",				/* AfterPrintStructures */
+  "{\\sf ",                        /* BeforePrintStructures */
+  " ",                                /* PrintStructureSeparator */
+  "} ",                                /* AfterPrintStructures */
 
-  "$<$",			/* StructureBeginPrefix */
-  "$>$",			/* StructureBeginSuffix */
+  "$<$",                        /* StructureBeginPrefix */
+  "$>$",                        /* StructureBeginSuffix */
 
-  " ",				/* StructureSeparator */
+  " ",                                /* StructureSeparator */
 
-  "$<$/",			/* StructureEndPrefix */
-  "$>$",			/* StructureEndSuffix */
+  "$<$/",                        /* StructureEndPrefix */
+  "$>$",                        /* StructureEndSuffix */
 
-  "",				/* BeforeToken */
-  " ",				/* TokenSeparator */
-  "/",				/* AttributeSeparator */
-  "",				/* AfterToken */
+  "",                                /* BeforeToken */
+  " ",                                /* TokenSeparator */
+  "/",                                /* AttributeSeparator */
+  "",                                /* AfterToken */
 
-  "",				/* BeforeField */
-  "",				/* FieldSeparator */
-  "",				/* AfterField */
+  "",                                /* BeforeField */
+  "",                                /* FieldSeparator */
+  "",                                /* AfterField */
 
-  "\\item ",			/* BeforeLine */
-  "\n",				/* AfterLine */
+  "\\item ",                        /* BeforeLine */
+  "\n",                                /* AfterLine */
 
-  "\\begin{itemize}\n",		/* BeforeConcordance */
-  "\\end{itemize}\n",		/* AfterConcordance */
+  "\\begin{itemize}\n",                /* BeforeConcordance */
+  "\\end{itemize}\n",                /* AfterConcordance */
 
   latex_convert_string,
   latex_print_field
@@ -82,34 +85,34 @@ LaTeXPrintDescriptionRecord = {
 
 PrintDescriptionRecord
 LaTeXTabularPrintDescriptionRecord = {
-  "{\\em %d:\\/} & ",		/* CPOSPrintFormat */
+  "{\\em %d:\\/} & ",                /* CPOSPrintFormat */
 
-  "{\\sf ",			/* BeforePrintStructures */
-  " ",				/* PrintStructureSeparator */
-  "} & ",			/* AfterPrintStructures */
+  "{\\sf ",                        /* BeforePrintStructures */
+  " ",                                /* PrintStructureSeparator */
+  "} & ",                        /* AfterPrintStructures */
 
-  "$<$",			/* StructureBeginPrefix */
-  "$>$",			/* StructureBeginSuffix */
+  "$<$",                        /* StructureBeginPrefix */
+  "$>$",                        /* StructureBeginSuffix */
 
-  " ",				/* StructureSeparator */
+  " ",                                /* StructureSeparator */
 
-  "$<$/",			/* StructureEndPrefix */
-  "$>$",			/* StructureEndSuffix */
+  "$<$/",                        /* StructureEndPrefix */
+  "$>$",                        /* StructureEndSuffix */
 
-  "",				/* BeforeToken */
-  " ",				/* TokenSeparator */
-  "/",				/* AttributeSeparator */
-  "",				/* AfterToken */
+  "",                                /* BeforeToken */
+  " ",                                /* TokenSeparator */
+  "/",                                /* AttributeSeparator */
+  "",                                /* AfterToken */
 
-  "",				/* BeforeField */
-  "",				/* FieldSeparator */
-  "",				/* AfterField */
+  "",                                /* BeforeField */
+  "",                                /* FieldSeparator */
+  "",                                /* AfterField */
 
-  "",				/* BeforeLine */
-  " \\\\\n",			/* AfterLine */
+  "",                                /* BeforeLine */
+  " \\\\\n",                        /* AfterLine */
 
-  "\\begin{tabular}\n",		/* BeforeConcordance */
-  "\\end{tabular}\n",	/* AfterConcordance */
+  "\\begin{tabular}\n",                /* BeforeConcordance */
+  "\\end{tabular}\n",        /* AfterConcordance */
 
   latex_convert_string,
   latex_print_field
@@ -129,15 +132,15 @@ latex_print_field(FieldType field, int at_end)
   case MatchField:
     if (GlobalPrintOptions.print_tabular) {
       if (at_end)
-	return "} & ";
+        return "} & ";
       else
-	return " & {\\bf ";
+        return " & {\\bf ";
     }
     else {
       if (at_end)
-	return "} ";
+        return "} ";
       else
-	return " {\\bf ";
+        return " {\\bf ";
     }
     break;
 
@@ -164,10 +167,10 @@ latex_print_field(FieldType field, int at_end)
 char *
 latex_convert_string(char *s)
 {
-  static char latex_s[MAX_LINE_LENGTH*2];
+  static char latex_s[CL_MAX_LINE_LENGTH*2];
   int p;
 
-  if (!s || strlen(s) >(MAX_LINE_LENGTH))
+  if (!s || strlen(s) >(CL_MAX_LINE_LENGTH))
     return NULL;
   
   for (p = 0; *s; s++) {
@@ -215,6 +218,13 @@ latex_convert_string(char *s)
   return latex_s;
 }
 
+/**
+ * Does nothing: we don't support printing alignment data in LaTeX mode.
+ *
+ * @param stream          Destination for the output.
+ * @param attribute_name  The name of the aligned corpus.
+ * @param line            Character data of the line of aligned-corpus data to print.
+ */
 void
 latex_print_aligned_line(FILE *stream, char *attribute_name, char *line)
 {
@@ -223,23 +233,23 @@ latex_print_aligned_line(FILE *stream, char *attribute_name, char *line)
 
 void latex_print_context(ContextDescriptor *cd, FILE *stream)
 {
-  char *s;
+  /* char *s; */
 
   fputs("{\\em Left display context:\\/}  & ", stream);
 
   switch(cd->left_type) {
   case CHAR_CONTEXT:
-    fprintf(stream, "%d characters", cd->left_width);
+    Rprintf("%d characters", cd->left_width);
     break;
   case WORD_CONTEXT:
-    fprintf(stream, "%d tokens", cd->left_width);
+    Rprintf("%d tokens", cd->left_width);
     break;
   case STRUC_CONTEXT:
-    fprintf(stream, "%d %s", cd->left_width, 
-	    cd->left_structure_name ? cd->left_structure_name : "???");
+    Rprintf("%d %s", cd->left_width, 
+            cd->left_structure_name ? cd->left_structure_name : "???");
     break;
   default:
-    s = "error";
+    /* s = "error"; */
     break;
   }
 
@@ -249,49 +259,58 @@ void latex_print_context(ContextDescriptor *cd, FILE *stream)
 
   switch(cd->right_type) {
   case CHAR_CONTEXT:
-    fprintf(stream, "%d characters", cd->right_width);
+    Rprintf("%d characters", cd->right_width);
     break;
   case WORD_CONTEXT:
-    fprintf(stream, "%d tokens", cd->right_width);
+    Rprintf("%d tokens", cd->right_width);
     break;
   case STRUC_CONTEXT:
-    fprintf(stream, "%d %s", cd->right_width, 
-	    cd->right_structure_name ? cd->right_structure_name : "???");
+    Rprintf("%d %s", cd->right_width, 
+            cd->right_structure_name ? cd->right_structure_name : "???");
     break;
   default:
-    s = "error";
+    /* s = "error"; */
     break;
   }
 
   fputs("\\\\\n", stream);
 }
 
-void latex_print_corpus_header(CorpusList *cl, 
-			       FILE *stream)
+void
+latex_print_corpus_header(CorpusList *cl,
+                          FILE *stream)
 {
   time_t now;
+#ifndef __MINGW__
   struct passwd *pwd = NULL;
+#endif
 
-  (void) time(&now);
+  time(&now);
   /*   pwd = getpwuid(geteuid()); */
   /* disabled because of incompatibilities between different Linux versions */
 
-  fprintf(stream,
-	  "{\\em This concordance was generated by:\\/}\n"
-	  "\\begin{quote}\\begin{tabular}{ll}\n"
-	  "{\\em User:\\/}      & %s (%s) \\\\\n"
-	  "{\\em Date:\\/}      & %s \\\\\n"
-	  "{\\em Corpus:\\/}    & %s \\\\\n"
-	  "                     & %s \\\\\n"
-	  "{\\em Subcorpus:\\/} & %s:%s \\\\\n"
-	  "{\\em Number of Matches:\\/} & %d \\\\\n",
-	  (pwd ? pwd->pw_name : "unknown"),
-	  (pwd ? pwd->pw_gecos  : "unknown"),
-	  ctime(&now),
-	  (cl->corpus && cl->corpus->registry_name ? cl->corpus->registry_name : "unknown"),
-	  (cl->corpus && cl->corpus->name ? cl->corpus->name : "unknown"),
-	  cl->mother_name, cl->name,
-	  cl->size);
+  Rprintf(
+          "{\\em This concordance was generated by:\\/}\n"
+          "\\begin{quote}\\begin{tabular}{ll}\n"
+          "{\\em User:\\/}      & %s (%s) \\\\\n"
+          "{\\em Date:\\/}      & %s \\\\\n"
+          "{\\em Corpus:\\/}    & %s \\\\\n"
+          "                     & %s \\\\\n"
+          "{\\em Subcorpus:\\/} & %s:%s \\\\\n"
+          "{\\em Number of Matches:\\/} & %d \\\\\n",
+
+#ifndef __MINGW__
+          (pwd ? pwd->pw_name : "unknown"),
+          (pwd ? pwd->pw_gecos  : "unknown"),
+#else
+          "<unknown>",
+          "<unknown>",
+#endif
+          ctime(&now),
+          (cl->corpus && cl->corpus->registry_name ? cl->corpus->registry_name : "unknown"),
+          (cl->corpus && cl->corpus->name ? cl->corpus->name : "unknown"),
+          cl->mother_name, cl->name,
+          cl->size);
   
   latex_print_context(&CD, stream);
 
@@ -299,13 +318,13 @@ void latex_print_corpus_header(CorpusList *cl,
 }
 
 void latex_print_output(CorpusList *cl, 
-			FILE *stream,
-			int interactive,
-			ContextDescriptor *cd,
-			int first, int last)
+                        FILE *stream,
+                        int interactive,
+                        ContextDescriptor *cd,
+                        int first, int last)
 {
   int line, real_line;
-  ConcLineField clf[NoField];	/* NoField is largest field code (not used by us) */
+  ConcLineField clf[NoField];        /* NoField is largest field code (not used by us) */
   PrintDescriptionRecord *pdr;
 
   ParsePrintOptions();
@@ -332,21 +351,21 @@ void latex_print_output(CorpusList *cl,
       int v = 0;
 
       for (l = cd->printStructureTags->list; l; l = l->next)
-	if (l->status) {
-	  v++;
-	  break;
-	}
+        if (l->status) {
+          v++;
+          break;
+        }
 
       if (v)
-	strcat(latex_tab_format, GlobalPrintOptions.print_border ? "l|" : "l");
+        strcat(latex_tab_format, GlobalPrintOptions.print_border ? "l|" : "l");
     }
 
     strcat(latex_tab_format, GlobalPrintOptions.print_border ? "r|l|l|" : "rll");
 
-    fprintf(stream, "\\begin{tabular}{%s}\n", latex_tab_format);
+    Rprintf("\\begin{tabular}{%s}\n", latex_tab_format);
 
     if (GlobalPrintOptions.print_border)
-      fprintf(stream, "\\hline\n");
+      Rprintf("\\hline\n");
   }
   else {
     pdr = &LaTeXPrintDescriptionRecord;
@@ -358,7 +377,7 @@ void latex_print_output(CorpusList *cl,
   if ((last >= cl->size) || (last < 0))
     last = cl->size - 1;
 
-  for (line = first; (line <= last) && (!broken_pipe); line++) {
+  for (line = first; (line <= last) && !cl_broken_pipe; line++) {
 
     if (cl->sortidx)
       real_line = cl->sortidx[line];
@@ -401,19 +420,19 @@ void latex_print_output(CorpusList *cl,
     {
       char *outstr;
       int dummy;
-	
+
       outstr = compose_kwic_line(cl->corpus, 
-				 cl->range[real_line].start, 
-				 cl->range[real_line].end,
-				 &CD, 
-				 &dummy,
-				 &dummy, &dummy,
-				 NULL, NULL, 
-				 NULL, 0, NULL,
-				 clf, NoField, /* NoField = # of entries in clf[] */
-				 ConcLineHorizontal, 
-				 pdr,
-				 0, NULL);
+                                 cl->range[real_line].start,
+                                 cl->range[real_line].end,
+                                 &CD,
+                                 &dummy,
+                                 &dummy, &dummy,
+                                 NULL, NULL,
+                                 NULL, 0, NULL,
+                                 clf, NoField, /* NoField = # of entries in clf[] */
+                                 ConcLineHorizontal,
+                                 pdr,
+                                 0, NULL);
       fputs(outstr, stream);
       free(outstr);
     }
@@ -422,15 +441,15 @@ void latex_print_output(CorpusList *cl,
       
     if (CD.alignedCorpora != NULL) {
       printAlignedStrings(cl->corpus, 
-			  &CD, 
-			  cl->range[real_line].start, 
-			  cl->range[real_line].end, 
-			  0,	/* ASCII print mode only */
-			  stream);
+                          &CD,
+                          cl->range[real_line].start,
+                          cl->range[real_line].end,
+                          0,        /* ASCII print mode only */
+                          stream);
     }
 
     if (GlobalPrintOptions.print_tabular && GlobalPrintOptions.print_border)
-      fprintf(stream, "\\hline\n");
+      Rprintf("\\hline\n");
   }
 
   fputs(pdr->AfterConcordance, stream);
@@ -450,9 +469,9 @@ latex_print_group(Group *group, int expand, FILE *fd)
   last_source_id = -999;
   nr_targets = 0;
 
-  fprintf(fd, "\\begin{tabular}{llr}\n");
+  Rprintf("\\begin{tabular}{llr}\n");
 
-  for (cell = 0; cell < group->nr_cells; cell++) {
+  for (cell = 0; (cell < group->nr_cells) && !cl_broken_pipe; cell++) {
 
     source_id = group->count_cells[cell].s;
     
@@ -466,11 +485,11 @@ latex_print_group(Group *group, int expand, FILE *fd)
     target_s = Group_id2str(group, target_id, 1);
     count     = group->count_cells[cell].freq;
     
-    fprintf(fd, " & %s & %d \\\\\n", 
-	    latex_convert_string(target_s), count);
+    Rprintf(" & %s & %d \\\\\n", 
+            latex_convert_string(target_s), count);
     
     nr_targets++;
   }
 
-  fprintf(fd, "\\end{tabular}\n");
+  Rprintf("\\end{tabular}\n");
 }
