@@ -1,11 +1,11 @@
 library(RcppCWB)
 
-testthat::context("cwb_cpos2lbound")
+testthat::context("cl_cpos2lbound")
 
 test_that(
   "cpos2lbound",
   {
-    cpos <- cwb_cpos2lbound(
+    cpos <- cl_cpos2lbound(
       corpus = "REUTERS",
       s_attribute = "places",
       cpos = 5L,
